==> pan-0.9.tar.bz2: pan-0.9/pan/thread-watch.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <glib.h>

#include "thread-watch.h"
#include "gnksa.h"
#include "prefs.h"
#include "util.h"

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

static GHashTable * hash = NULL;

static gboolean dirty = FALSE;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PRIVATE
************/

/*****
******
*****/

static gchar*
get_filename (void)
{
	return g_strdup_printf ("%s/threads.txt", data_dir);
}

/*****
******
*****/

typedef struct
{
	size_t start_time;
	int state;
}
RecordStruct;

static void
thread_init_module (void)
{
	if (hash == NULL)
	{
		const time_t current_time = time(0);
		RecordStruct * data;
		gchar * filename;
		GArray * file;
		gchar ** v;
		gint i;

		/* create the hashtable */
		hash = g_hash_table_new (g_str_hash, g_str_equal);
		dirty = FALSE;

		/* load the file */
		filename = get_filename ();
		file = read_file (filename);
		v = file==NULL ? NULL : g_strsplit (file->data, "\n", -1);
		for (i=0; v!=NULL && v[i]!=NULL; ++i)
		{
			int state;
			time_t start_time;
			gchar * id;
			const gchar * march = v[i];
			time_t age_secs;

			/* skip comments */
			if (*march=='#')
				continue;

			/* get info */
			state      = get_next_token_int   (march, ',', &march);
			start_time = get_next_token_ulong (march, ',', &march);
			id         = get_next_token_str   (march, ',', &march);

			/* age out old records */
			age_secs = current_time - start_time;
			if ((age_secs/86400) > watched_thread_span_days) {
				dirty = TRUE;
				g_free (id);
				continue;
			}

			/* insert to the hash table */
			data = g_new (RecordStruct, 1);
			data->start_time = start_time;
			data->state = state;
			g_hash_table_insert (hash, id, data);
		}

		/* cleanup */
		g_free (filename);
		if (file != NULL)
			g_array_free (file, TRUE);
		g_strfreev (v);
	}
}

/*****
******
*****/

static void
hash_freekeyval (gpointer key, gpointer value, gpointer user_data)
{
	g_free (key);
	g_free (value);
}
static void
free_hash (void)
{
	if (hash != NULL)
	{
		g_hash_table_foreach (hash, hash_freekeyval, NULL);
		g_hash_table_destroy (hash);
		hash = NULL;
	}
}

/*****
******
*****/

static void
thread_write_line (gpointer key, gpointer value, gpointer user_data)
{
	const RecordStruct * data = (const RecordStruct*) value;
	if (data->state != THREAD_NOTHING)
	{
		const gchar * id = (const gchar*) key;
		FILE * fp = (FILE*)user_data;

		fprintf (fp, "%d,%lu,%s\n",
			data->state,
			(gulong)data->start_time,
			id);
	}
}

/*****
******
*****/


/************
*************  PUBLIC
************/

gint
thread_get_state (const Article * a)
{
	gchar * id = NULL;
	gint retval = THREAD_NOTHING;
	gpointer data = NULL;

	/* make sure module's ready */
	if (hash == NULL)
		thread_init_module ();

	/* get the thread state */
	if (a != NULL)
		id = article_get_thread_message_id (a);
	if (id != NULL)
		data = g_hash_table_lookup (hash, id);
	if (data != NULL)
		retval = ((RecordStruct*)data)->state;

	/* cleanup */
	g_free (id);

	return retval;
}

void
thread_set_state (const Article * a, int state)
{
	gchar * id;

	/* sanity checks */
	g_return_if_fail (a!=NULL);
	g_assert (is_nonempty_string(a->message_id));

	/* make sure module's ready */
	if (hash == NULL)
		thread_init_module ();

	/* check that thread's message id */
       	id = article_get_thread_message_id (a);
	if (is_nonempty_string(id))
	{
		gpointer val = g_hash_table_lookup (hash, id);

		/* update/insert the thread's state */
		if (val != NULL)
		{
			RecordStruct * data = (RecordStruct*) val;
			data->start_time = time(0);
			data->state = state;
			g_free (id);
		}
		else
		{
			RecordStruct * data = g_new0 (RecordStruct, 1);
			data->start_time = time(0);
			data->state = state;
			g_hash_table_insert (hash, id, data);
		}
		dirty = TRUE;
	}
}

void
thread_shutdown_module (void)
{
	if (dirty)
	{
		/* write the thread states to disk */
		gchar * filename = get_filename ();
		FILE * fp = fopen (filename, "w+");
		fprintf (fp, "# one record per line, comma-delimited fields\n");
		fprintf (fp, "# each record has three fields:\n");
		fprintf (fp, "#    state (0=nothing,1=watched,2=ignored)\n");
		fprintf (fp, "#    start time (in secs since Jan 1, 1970)\n");
		fprintf (fp, "#    message-id of parent thread\n");
		g_hash_table_foreach (hash, thread_write_line, fp);
		fclose (fp);
		g_free (filename);
		dirty = FALSE;
	}

	free_hash ();
}

==> pan-0.9.1.tar.bz2: pan-0.9.1/pan/thread-watch.c <==
/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*********************
**********************  Includes
*********************/

#include <config.h>

#include <glib.h>

#include "thread-watch.h"
#include "gnksa.h"
#include "prefs.h"
#include "util.h"

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

static GHashTable * hash = NULL;

static gboolean dirty = FALSE;

static guint hash_size = 0;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PRIVATE
************/

/*****
******
*****/

static gchar*
get_filename (void)
{
	return g_strdup_printf ("%s/threads.txt", data_dir);
}

/*****
******
*****/

typedef struct
{
	size_t start_time;
	int state;
}
RecordStruct;

static void
thread_init_module (void)
{
	if (hash == NULL)
	{
		const time_t current_time = time(0);
		RecordStruct * data;
		gchar * filename;
		GArray * file;
		gchar ** v;
		gint i;

		/* create the hashtable */
		hash = g_hash_table_new (g_str_hash, g_str_equal);
		hash_size = 0;
		dirty = FALSE;

		/* load the file */
		filename = get_filename ();
		file = read_file (filename);
		v = file==NULL ? NULL : g_strsplit (file->data, "\n", -1);
		for (i=0; v!=NULL && v[i]!=NULL; ++i)
		{
			int state;
			time_t start_time;
			gchar * id;
			const gchar * march = v[i];
			time_t age_secs;

			/* skip comments */
			if (*march=='#')
				continue;

			/* get info */
			state      = get_next_token_int   (march, ',', &march);
			start_time = get_next_token_ulong (march, ',', &march);
			id         = get_next_token_str   (march, ',', &march);

			/* age out threads > one month old */
			age_secs = current_time - start_time;
			if ((age_secs/86400) > watched_thread_span_days) {
				dirty = TRUE;
				g_free (id);
				continue;
			}

			/* insert to the hash table */
			data = g_new (RecordStruct, 1);
			data->start_time = start_time;
			data->state = state;
			g_hash_table_insert (hash, id, data);
			++hash_size;
		}

		/* cleanup */
		g_free (filename);
		if (file != NULL)
			g_array_free (file, TRUE);
		g_strfreev (v);
	}
}

/*****
******
*****/

static void
hash_freekeyval (gpointer key, gpointer value, gpointer user_data)
{
	g_free (key);
	g_free (value);
}
static void
free_hash (void)
{
	if (hash != NULL)
	{
		g_hash_table_foreach (hash, hash_freekeyval, NULL);
		g_hash_table_destroy (hash);
		hash = NULL;
	}

	hash_size = 0;
}

/*****
******
*****/

static void
thread_write_line (gpointer key, gpointer value, gpointer user_data)
{
	const RecordStruct * data = (const RecordStruct*) value;
	if (data->state != THREAD_NOTHING)
	{
		const gchar * id = (const gchar*) key;
		FILE * fp = (FILE*)user_data;

		fprintf (fp, "%d,%lu,%s\n",
			data->state,
			(gulong)data->start_time,
			id);
	}
}

/*****
******
*****/


/************
*************  PUBLIC
************/

gint
thread_get_state (const Article * a)
{
	gint retval = THREAD_NOTHING;

	/* make sure module's ready */
	if (hash == NULL)
		thread_init_module ();

	/* get the thread state */
	if (hash_size != 0)
	{
		if (a != NULL)
		{
			gchar * id = article_get_thread_message_id (a);

			if (id != NULL)
			{
				gpointer data = g_hash_table_lookup (hash, id);

				if (data != NULL)
				{
					retval = ((RecordStruct*)data)->state;
				}

				g_free (id);
			}
		}
	}

	return retval;
}

void
thread_set_state (const Article * a, int state)
{
	gchar * id;

	/* sanity checks */
	g_return_if_fail (a!=NULL);
	g_assert (is_nonempty_string(a->message_id));

	/* make sure module's ready */
	if (hash == NULL)
		thread_init_module ();

	/* check that thread's message id */
       	id = article_get_thread_message_id (a);
	if (is_nonempty_string(id))
	{
		gpointer val = g_hash_table_lookup (hash, id);

		/* update/insert the thread's state */
		if (val != NULL)
		{
			RecordStruct * data = (RecordStruct*) val;
			data->start_time = time(0);
			data->state = state;
			g_free (id);
		}
		else
		{
			RecordStruct * data = g_new0 (RecordStruct, 1);
			data->start_time = time(0);
			data->state = state;
			g_hash_table_insert (hash, id, data);
			++hash_size;
		}
		dirty = TRUE;
	}
}

void
thread_shutdown_module (void)
{
	if (dirty)
	{
		/* write the thread states to disk */
		gchar * filename = get_filename ();
		FILE * fp = fopen (filename, "w+");
		fprintf (fp, "# one record per line, comma-delimited fields\n");
		fprintf (fp, "# each record has three fields:\n");
		fprintf (fp, "#    state (0=nothing,1=watched,2=ignored)\n");
		fprintf (fp, "#    start time (in secs since Jan 1, 1970)\n");
		fprintf (fp, "#    message-id of parent thread\n");
		g_hash_table_foreach (hash, thread_write_line, fp);
		fclose (fp);
		g_free (filename);
		dirty = FALSE;
	}

	free_hash ();
}
